﻿Imports System.Threading.Tasks
Imports System.Web.Mvc
Imports Microsoft.AspNet.Identity
Imports Microsoft.AspNet.Identity.Owin
Imports Microsoft.Owin.Security
Imports MVCAppWithDocDB.Models

Namespace MVCAppWithDocDB.Controllers
	<Authorize>
	Public Class ManageController
		Inherits Controller
		Private _signInManager As ApplicationSignInManager
		Private _userManager As ApplicationUserManager

		Public Sub New()
		End Sub

		Public Sub New(ByVal userManager As ApplicationUserManager, ByVal signInManager As ApplicationSignInManager)
			Me.UserManager = userManager
			Me.SignInManager = signInManager
		End Sub

		Public Property SignInManager As ApplicationSignInManager
			Get
				Return If(_signInManager, HttpContext.GetOwinContext.Get(Of ApplicationSignInManager)())
			End Get
			Private Set(ByVal value As ApplicationSignInManager)
				_signInManager = value
			End Set
		End Property

		Public Property UserManager As ApplicationUserManager
			Get
				Return If(_userManager, HttpContext.GetOwinContext.GetUserManager(Of ApplicationUserManager)())
			End Get
			Private Set(ByVal value As ApplicationUserManager)
				_userManager = value
			End Set
		End Property

		'
		' GET: /Manage/Index
		Public async Function Index(ByVal message? As ManageMessageId) As Task(Of ActionResult)
			ViewBag.StatusMessage = If(message = ManageMessageId.ChangePasswordSuccess, "Your password has been changed.", If(message = ManageMessageId.SetPasswordSuccess, "Your password has been set.", If(message = ManageMessageId.SetTwoFactorSuccess, "Your two-factor authentication provider has been set.", If(message = ManageMessageId.Error, "An error has occurred.", If(message = ManageMessageId.AddPhoneSuccess, "Your phone number was added.", If(message = ManageMessageId.RemovePhoneSuccess, "Your phone number was removed.", ""))))))

			Dim userId = User.Identity.GetUserId
			Dim model = New IndexViewModel With {.HasPassword = HasPassword(), .PhoneNumber = await UserManager.GetPhoneNumberAsync(userId), .TwoFactor = await UserManager.GetTwoFactorEnabledAsync(userId), .Logins = await UserManager.GetLoginsAsync(userId), .BrowserRemembered = await AuthenticationManager.TwoFactorBrowserRememberedAsync(userId)}
			Return View(model)
		End Function

		'
		' POST: /Manage/RemoveLogin
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function RemoveLogin(ByVal loginProvider As String, ByVal providerKey As String) As Task(Of ActionResult)
			Dim message? As ManageMessageId
			Dim result = Await UserManager.RemoveLoginAsync(User.Identity.GetUserId, New UserLoginInfo(loginProvider, providerKey))
			If result.Succeeded Then
                Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
                If userInfo IsNot Nothing Then
                    Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
                End If
				message = ManageMessageId.RemoveLoginSuccess
			Else
				message = ManageMessageId.Error
			End If
			Return RedirectToAction("ManageLogins", New With {Key .Message = message})
		End Function

		'
		' GET: /Manage/AddPhoneNumber
		Public Function AddPhoneNumber() As ActionResult
			Return View()
		End Function

		'
		' POST: /Manage/AddPhoneNumber
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function AddPhoneNumber(ByVal model As AddPhoneNumberViewModel) As Task(Of ActionResult)
			If Not ModelState.IsValid Then
				Return View(model)
			End If
			' Generate the token and send it
			Dim code = Await UserManager.GenerateChangePhoneNumberTokenAsync(User.Identity.GetUserId, model.Number)
			If UserManager.SmsService IsNot Nothing Then
				Dim message = New IdentityMessage With {.Destination = model.Number, .Body = "Your security code is: " & code}
				await UserManager.SmsService.SendAsync(message)
			End If
			Return RedirectToAction("VerifyPhoneNumber", New With {Key .PhoneNumber = model.Number})
		End Function

		'
		' POST: /Manage/EnableTwoFactorAuthentication
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function EnableTwoFactorAuthentication() As Task(Of ActionResult)
            Await UserManager.SetTwoFactorEnabledAsync(User.Identity.GetUserId, True)
            Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
            If userInfo IsNot Nothing Then
                Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
            End If
			Return RedirectToAction("Index", "Manage")
		End Function

		'
		' POST: /Manage/DisableTwoFactorAuthentication
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function DisableTwoFactorAuthentication() As Task(Of ActionResult)
            Await UserManager.SetTwoFactorEnabledAsync(User.Identity.GetUserId, False)
            Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
            If userInfo IsNot Nothing Then
                Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
            End If
			Return RedirectToAction("Index", "Manage")
		End Function

		'
		' GET: /Manage/VerifyPhoneNumber
		Public async Function VerifyPhoneNumber(ByVal phoneNumber As String) As Task(Of ActionResult)
			Dim code = Await UserManager.GenerateChangePhoneNumberTokenAsync(User.Identity.GetUserId, phoneNumber)
			' Send an SMS through the SMS provider to verify the phone number
			Return If(phoneNumber Is Nothing, View("Error"), View(New VerifyPhoneNumberViewModel With {.PhoneNumber = phoneNumber}))
		End Function

		'
		' POST: /Manage/VerifyPhoneNumber
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function VerifyPhoneNumber(ByVal model As VerifyPhoneNumberViewModel) As Task(Of ActionResult)
			If Not ModelState.IsValid Then
				Return View(model)
			End If
			Dim result = Await UserManager.ChangePhoneNumberAsync(User.Identity.GetUserId, model.PhoneNumber, model.Code)
			If result.Succeeded Then
                Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
                If userInfo IsNot Nothing Then
                    Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
                End If
				Return RedirectToAction("Index", New With {Key .Message = ManageMessageId.AddPhoneSuccess})
			End If
			' If we got this far, something failed, redisplay form
			ModelState.AddModelError("", "Failed to verify phone")
			Return View(model)
		End Function

		'
		' GET: /Manage/RemovePhoneNumber
		Public async Function RemovePhoneNumber() As Task(Of ActionResult)
            Dim result = Await UserManager.SetPhoneNumberAsync(User.Identity.GetUserId, Nothing)
			If Not result.Succeeded Then
				Return RedirectToAction("Index", New With {Key .Message = ManageMessageId.Error})
			End If
            Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
            If userInfo IsNot Nothing Then
                Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
            End If
			Return RedirectToAction("Index", New With {Key .Message = ManageMessageId.RemovePhoneSuccess})
		End Function

		'
		' GET: /Manage/ChangePassword
		Public Function ChangePassword() As ActionResult
			Return View()
		End Function

		'
		' POST: /Manage/ChangePassword
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function ChangePassword(ByVal model As ChangePasswordViewModel) As Task(Of ActionResult)
			If Not ModelState.IsValid Then
				Return View(model)
			End If
			Dim result = Await UserManager.ChangePasswordAsync(User.Identity.GetUserId, model.OldPassword, model.NewPassword)
			If result.Succeeded Then
                Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
                If userInfo IsNot Nothing Then
                    Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
                End If
				Return RedirectToAction("Index", New With {Key .Message = ManageMessageId.ChangePasswordSuccess})
			End If
			AddErrors(result)
			Return View(model)
		End Function

		'
		' GET: /Manage/SetPassword
		Public Function SetPassword() As ActionResult
			Return View()
		End Function

		'
		' POST: /Manage/SetPassword
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function SetPassword(ByVal model As SetPasswordViewModel) As Task(Of ActionResult)
			If ModelState.IsValid Then
				Dim result = Await UserManager.AddPasswordAsync(User.Identity.GetUserId, model.NewPassword)
				If result.Succeeded Then
                    Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
                    If userInfo IsNot Nothing Then
                        Await SignInManager.SignInAsync(userInfo, isPersistent:=False, rememberBrowser:=False)
                    End If
					Return RedirectToAction("Index", New With {Key .Message = ManageMessageId.SetPasswordSuccess})
				End If
				AddErrors(result)
			End If

			' If we got this far, something failed, redisplay form
			Return View(model)
		End Function

		'
		' GET: /Manage/ManageLogins
		Public async Function ManageLogins(ByVal message? As ManageMessageId) As Task(Of ActionResult)
			ViewBag.StatusMessage = If(message = ManageMessageId.RemoveLoginSuccess, "The external login was removed.", If(message = ManageMessageId.Error, "An error has occurred.", ""))
            Dim userInfo = Await UserManager.FindByIdAsync(User.Identity.GetUserId)
            If userInfo Is Nothing Then
                Return View("Error")
            End If
            Dim userLogins = Await UserManager.GetLoginsAsync(User.Identity.GetUserId)
			Dim otherLogins = AuthenticationManager.GetExternalAuthenticationTypes.Where(Function(auth) userLogins.All(Function(ul) auth.AuthenticationType <> ul.LoginProvider)).ToList
            ViewBag.ShowRemoveButton = userInfo.PasswordHash IsNot Nothing OrElse userLogins.Count > 1
			Return View(New ManageLoginsViewModel With {.CurrentLogins = userLogins, .OtherLogins = otherLogins})
		End Function

		'
		' POST: /Manage/LinkLogin
		<HttpPost, ValidateAntiForgeryToken>
		Public Function LinkLogin(ByVal provider As String) As ActionResult
			' Request a redirect to the external login provider to link a login for the current user
			Return New AccountController.ChallengeResult(provider, Url.Action("LinkLoginCallback", "Manage"), User.Identity.GetUserId)
		End Function

		'
		' GET: /Manage/LinkLoginCallback
		Public async Function LinkLoginCallback() As Task(Of ActionResult)
			Dim loginInfo = Await AuthenticationManager.GetExternalLoginInfoAsync(XsrfKey, User.Identity.GetUserId)
			If loginInfo Is Nothing Then
				Return RedirectToAction("ManageLogins", New With {Key .Message = ManageMessageId.Error})
			End If
			Dim result = Await UserManager.AddLoginAsync(User.Identity.GetUserId, loginInfo.Login)
			Return If(result.Succeeded, RedirectToAction("ManageLogins"), RedirectToAction("ManageLogins", New With {Key .Message = ManageMessageId.Error}))
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			If disposing AndAlso _userManager IsNot Nothing Then
				_userManager.Dispose()
				_userManager = Nothing
			End If

			MyBase.Dispose(disposing)
		End Sub

#Region "Helpers"
		' Used for XSRF protection when adding external logins
		Private Const XsrfKey As String = "XsrfId"

		Private ReadOnly Property AuthenticationManager As IAuthenticationManager
			Get
				Return HttpContext.GetOwinContext.Authentication
			End Get
		End Property

		Private Sub AddErrors(ByVal result As IdentityResult)
			For Each [error] In result.Errors
				ModelState.AddModelError("", [error])
			Next [error]
		End Sub

		Private Function HasPassword() As Boolean
            Dim userInfo = UserManager.FindById(User.Identity.GetUserId)
            If userInfo IsNot Nothing Then
                Return userInfo.PasswordHash IsNot Nothing
            End If
			Return False
		End Function

		Private Function HasPhoneNumber() As Boolean
            Dim userInfo = UserManager.FindById(User.Identity.GetUserId)
            If userInfo IsNot Nothing Then
                Return userInfo.PhoneNumber IsNot Nothing
            End If
			Return False
		End Function

		Public Enum ManageMessageId
			AddPhoneSuccess
			ChangePasswordSuccess
			SetTwoFactorSuccess
			SetPasswordSuccess
			RemoveLoginSuccess
			RemovePhoneSuccess
			[Error]
		End Enum

#End Region
	End Class
End Namespace